@@ -1,7 +1,7 @@
+#include <config.h>
 #include "ev-pixbuf-cache.h"
 #include "ev-job-queue.h"
 #include "ev-page-cache.h"
-#include "ev-selection.h"
 #include "ev-document-images.h"
 #include "ev-document-forms.h"
 #include "ev-image.h"
@@ -11,6 +11,7 @@ typedef struct _CacheJobInfo
 {
 	EvJob *job;
 	EvRenderContext *rc;
+	gboolean page_ready;
 
 	/* Region of the page that needs to be drawn */
 	GdkRegion *region; 
@@ -25,9 +26,10 @@ typedef struct _CacheJobInfo
 	/* Selection data. 
 	 * Selection_points are the coordinates encapsulated in selection.
 	 * target_points is the target selection size. */
-	EvRectangle selection_points;
-	EvRectangle target_points;
-	gboolean    points_set;
+	EvRectangle      selection_points;
+	EvRectangle      target_points;
+	EvSelectionStyle selection_style;
+	gboolean         points_set;
 	
 	cairo_surface_t *selection;
 	GdkRegion *selection_region;
@@ -73,6 +75,8 @@ static void          ev_pixbuf_cache_init       (EvPixbufCache      *pixbuf_cach
 static void          ev_pixbuf_cache_class_init (EvPixbufCacheClass *pixbuf_cache);
 static void          ev_pixbuf_cache_finalize   (GObject            *object);
 static void          ev_pixbuf_cache_dispose    (GObject            *object);
+static void          job_page_ready_cb          (EvJob              *job,
+						 EvPixbufCache      *pixbuf_cache);
 static void          job_finished_cb            (EvJob              *job,
 						 EvPixbufCache      *pixbuf_cache);
 static CacheJobInfo *find_job_cache             (EvPixbufCache      *pixbuf_cache,
@@ -80,6 +84,9 @@ static CacheJobInfo *find_job_cache             (EvPixbufCache      *pixbuf_cach
 static void          copy_job_to_job_info       (EvJobRender        *job_render,
 						 CacheJobInfo       *job_info,
 						 EvPixbufCache      *pixbuf_cache);
+static void          copy_job_page_and_selection_to_job_info (EvJobRender        *job_render,
+							      CacheJobInfo       *job_info,
+							      EvPixbufCache      *pixbuf_cache);
 static gboolean      new_selection_surface_needed(EvPixbufCache      *pixbuf_cache,
 						  CacheJobInfo       *job_info,
 						  gint                page,
@@ -150,6 +157,9 @@ dispose_cache_job_info (CacheJobInfo *job_info,
 	if (job_info == NULL)
 		return;
 	if (job_info->job) {
+		g_signal_handlers_disconnect_by_func (job_info->job,
+						      G_CALLBACK (job_page_ready_cb),
+						      data);
 		g_signal_handlers_disconnect_by_func (job_info->job,
 						      G_CALLBACK (job_finished_cb),
 						      data);
@@ -232,6 +242,26 @@ ev_pixbuf_cache_new (GtkWidget  *view,
 	return pixbuf_cache;
 }
 
+static void
+job_page_ready_cb (EvJob         *job,
+		   EvPixbufCache *pixbuf_cache)
+{
+	CacheJobInfo *job_info;
+	EvJobRender *job_render = EV_JOB_RENDER (job);
+
+	/* If the job is outside of our interest, we silently discard it */
+	if ((job_render->rc->page < (pixbuf_cache->start_page - pixbuf_cache->preload_cache_size)) ||
+	    (job_render->rc->page > (pixbuf_cache->end_page + pixbuf_cache->preload_cache_size))) {
+		g_object_unref (job);
+		return;
+	}
+
+	job_info = find_job_cache (pixbuf_cache, job_render->rc->page);
+
+	copy_job_page_and_selection_to_job_info (job_render, job_info, pixbuf_cache);
+	g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0, job_info->region);
+}
+
 static void
 job_finished_cb (EvJob         *job,
 		 EvPixbufCache *pixbuf_cache)
@@ -245,11 +275,9 @@ job_finished_cb (EvJob         *job,
 		g_object_unref (job);
 		return;
 	}
-	
-	job_info = find_job_cache (pixbuf_cache, job_render->rc->page);
 
+	job_info = find_job_cache (pixbuf_cache, job_render->rc->page);
 	copy_job_to_job_info (job_render, job_info, pixbuf_cache);
-	g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0, job_info->region);
 }
 
 /* This checks a job to see if the job would generate the right sized pixbuf
@@ -279,6 +307,9 @@ check_job_size_and_unref (EvPixbufCache *pixbuf_cache,
 	    height == EV_JOB_RENDER (job_info->job)->target_height)
 		return;
 
+	g_signal_handlers_disconnect_by_func (job_info->job,
+					      G_CALLBACK (job_page_ready_cb),
+					      pixbuf_cache);
 	g_signal_handlers_disconnect_by_func (job_info->job,
 					      G_CALLBACK (job_finished_cb),
 					      pixbuf_cache);
@@ -419,13 +450,10 @@ ev_pixbuf_cache_update_range (EvPixbufCache *pixbuf_cache,
 }
 
 static void
-copy_job_to_job_info (EvJobRender   *job_render,
-		      CacheJobInfo  *job_info,
-		      EvPixbufCache *pixbuf_cache)
+copy_job_page_and_selection_to_job_info (EvJobRender   *job_render,
+					 CacheJobInfo  *job_info,
+					 EvPixbufCache *pixbuf_cache)
 {
-
-	job_info->points_set = FALSE;
-
 	if (job_info->surface) {
 		cairo_surface_destroy (job_info->surface);
 	}
@@ -436,6 +464,38 @@ copy_job_to_job_info (EvJobRender   *job_render,
 	}
 	job_info->rc = g_object_ref (job_render->rc);
 
+	job_info->points_set = FALSE;
+	if (job_render->include_selection) {
+		if (job_info->selection) {
+			cairo_surface_destroy (job_info->selection);
+			job_info->selection = NULL;
+		}
+		if (job_info->selection_region) {
+			gdk_region_destroy (job_info->selection_region);
+			job_info->selection_region = NULL;
+		}
+		
+		job_info->selection_points = job_render->selection_points;
+		job_info->selection_region = gdk_region_copy (job_render->selection_region);
+		job_info->selection = cairo_surface_reference (job_render->selection);
+		g_assert (job_info->selection_points.x1 >= 0);
+		job_info->points_set = TRUE;
+	}
+
+	if (job_info->job) {
+		g_signal_handlers_disconnect_by_func (job_info->job,
+						      G_CALLBACK (job_page_ready_cb),
+						      pixbuf_cache);
+	}
+
+	job_info->page_ready = TRUE;
+}
+
+static void
+copy_job_to_job_info (EvJobRender   *job_render,
+		      CacheJobInfo  *job_info,
+		      EvPixbufCache *pixbuf_cache)
+{
 	if (job_render->include_links) {
 		if (job_info->link_mapping)
 			ev_link_mapping_free (job_info->link_mapping);
@@ -460,23 +520,6 @@ copy_job_to_job_info (EvJobRender   *job_render,
 		job_info->text_mapping = job_render->text_mapping;
 	}
 
-	if (job_render->include_selection) {
-		if (job_info->selection) {
-			cairo_surface_destroy (job_info->selection);
-			job_info->selection = NULL;
-		}
-		if (job_info->selection_region) {
-			gdk_region_destroy (job_info->selection_region);
-			job_info->selection_region = NULL;
-		}
-		
-		job_info->selection_points = job_render->selection_points;
-		job_info->selection_region = gdk_region_copy (job_render->selection_region);
-		job_info->selection = cairo_surface_reference (job_render->selection);
-		g_assert (job_info->selection_points.x1 >= 0);
-		job_info->points_set = TRUE;
-	}
-
 	if (job_info->job) {
 		g_signal_handlers_disconnect_by_func (job_info->job,
 						      G_CALLBACK (job_finished_cb),
@@ -568,9 +611,11 @@ add_job (EvPixbufCache *pixbuf_cache,
 	gboolean include_links = FALSE;
 	gboolean include_text = FALSE;
 	gboolean include_selection = FALSE;
-	gboolean include_images = TRUE;
+	gboolean include_images = FALSE;
 	gboolean include_forms = FALSE;
 	GdkColor *text, *base;
+
+	job_info->page_ready = FALSE;
 	
 	if (job_info->rc == NULL) {
 		job_info->rc = ev_render_context_new (rotation, page, scale);
@@ -605,6 +650,7 @@ add_job (EvPixbufCache *pixbuf_cache,
 					   job_info->rc,
 					   width, height,
 					   &(job_info->target_points),
+					   job_info->selection_style,
 					   text, base,
 					   include_forms,
 					   include_links,
@@ -612,7 +658,10 @@ add_job (EvPixbufCache *pixbuf_cache,
 					   include_text,
 					   include_selection);
 	ev_job_queue_add_job (job_info->job, priority);
-	g_signal_connect (job_info->job, "finished",
+	g_signal_connect (G_OBJECT (job_info->job), "page-ready",
+			  G_CALLBACK (job_page_ready_cb),
+			  pixbuf_cache);
+	g_signal_connect (G_OBJECT (job_info->job), "finished",
 			  G_CALLBACK (job_finished_cb),
 			  pixbuf_cache);
 }
@@ -729,10 +778,13 @@ ev_pixbuf_cache_get_surface (EvPixbufCache *pixbuf_cache,
 	if (job_info == NULL)
 		return NULL;
 
+	if (job_info->page_ready)
+		return job_info->surface;
+	
 	/* We don't need to wait for the idle to handle the callback */
 	if (job_info->job &&
-	    EV_JOB (job_info->job)->finished) {
-		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
+	    EV_JOB_RENDER (job_info->job)->page_ready) {
+		copy_job_page_and_selection_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
 		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0, job_info->region);
 	}
 
@@ -753,7 +805,6 @@ ev_pixbuf_cache_get_link_mapping (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	    EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
-		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0, job_info->region);
 	}
 
 	return job_info->link_mapping;
@@ -776,7 +827,6 @@ ev_pixbuf_cache_get_image_mapping (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	    EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
-		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0, job_info->region);
 	}
 
 	return job_info->image_mapping;
@@ -799,7 +849,6 @@ ev_pixbuf_cache_get_form_field_mapping (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	   EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER(job_info->job), job_info, pixbuf_cache);
-		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0, job_info->region);
 	}
 	
 	return job_info->form_field_mapping;
@@ -863,7 +912,6 @@ ev_pixbuf_cache_get_text_mapping (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	    EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
-		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0, job_info->region);
 	}
 	
 	return job_info->text_mapping;
@@ -978,6 +1026,7 @@ ev_pixbuf_cache_get_selection_surface (EvPixbufCache  *pixbuf_cache,
 		job_info->selection_region =
 			ev_selection_get_selection_region (EV_SELECTION (pixbuf_cache->document),
 							   job_info->rc,
+							   job_info->selection_style,
 							   &(job_info->target_points));
 
 		gtk_widget_ensure_style (pixbuf_cache->view);
@@ -988,6 +1037,7 @@ ev_pixbuf_cache_get_selection_surface (EvPixbufCache  *pixbuf_cache,
 					       job_info->rc, &(job_info->selection),
 					       &(job_info->target_points),
 					       old_points,
+					       job_info->selection_style,
 					       text, base);
 		job_info->selection_points = job_info->target_points;
 		ev_document_doc_mutex_unlock ();
@@ -1003,6 +1053,7 @@ update_job_selection (CacheJobInfo    *job_info,
 {
 	job_info->points_set = TRUE;		
 	job_info->target_points = selection->rect;
+	job_info->selection_style = selection->style;
 }
 
 static void
